function varargout = blobchooser(varargin)

% blobchooser
%	Developed by Ilya Valmianski
%	ivalmian@ucsd.edu
%
%	This is a GUI which helps annotate training data for second step learning. As input it can either open raw
%	 tif/mpd file or .mat files it has generated previously. It's outputare .mat files which can be passed on to 
%	PrepareBlobTraining.m, which will use them to generate output.data and output.spec files.
%
%	Note that blobchooser uses a pixel-wise classifier to generate examples for annotation. The particular pixel-wise
%	classifier is specified in generateObjStruct.m line 25. The default value of that line is
%
%	[scores filters meanim]=scoreIm(stackin,@firststage,1);
%
%	If you want to use your own pixel-wise classifier, replace firststage,
%	with the name of your classifier. You can either generate you own, or 
%   use one of the other classifiers present in that directory, ie: 
%   calcScore_generous
%   calcScoreLogLossResampled
%   calcScoreRSRobustBoost16pEpsilon
%   calcScoreRSRobustBoost20pEpsilon
%
%	Also, you will need to place the classifier file somewhere in the path,
%   preferably into the getImages directory.
%
%	Note, this GUI has keyboard short cuts:
%
%     	Key 'a'
%           Accept (mark as positive example)
%       Key 'r'
%            Reject (mark as negative example)
%       Key 'd'
%           Don't judge (don't mark as example)
%       Key 'n'
 %           Select next blob
 %     Key 'p'
 %          Select previous blob

gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @blobchooser_OpeningFcn, ...
                   'gui_OutputFcn',  @blobchooser_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before blobchooser is made visible.
function blobchooser_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to blobchooser (see VARARGIN)

% Choose default command line output for blobchooser
handles.output = hObject;

% Generate data variable.

handles.objData=struct('Init',0, ...         %dataset initialized
                        'meanIm',[],...     %mean image
                        'scoreIm',[],...    %score image
                        'normmeanIm',[],... %normalized mean image
                        'choiceMap',[],...  %choice map image
                        'boosterData',[],...%data as required for learning
                        'objNum',0,...      %current object number
                        'objTot',0,...      %total objects on this level
                        'threshNum',NaN,... %current theshold
                        'threshList',[]);   %list of applicable threshold

%whenever any key is pressed, myFunction is called
set(handles.figure1,'KeyPressFcn',@keyPressFunc);

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes blobchooser wait for user response (see UIRESUME)
% uiwait(handles.figure1);

% --- key press shortcut
function keyPressFunc(hObject,evnt)
handles=guidata(gcf);

for x=1:length(evnt.Key)
    switch(evnt.Key(x))
        case 'a'
            accept_button_Callback(hObject, evnt, handles);
        case 'r'
            reject_button_Callback(hObject, evnt, handles);
        case 'd'
            djudge_button_Callback(hObject, evnt, handles);
        case 'n'
            nblob_button_Callback(hObject, evnt, handles);
        case 'p'
            pblob_button_Callback(hObject, evnt, handles);
    end
end

% --- Outputs from this function are returned to the command line.
function varargout = blobchooser_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


%------- BUTTONS ------%

% --- Executes on button press in save_button.
function save_button_Callback(hObject, eventdata, handles)
% hObject    handle to save_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[fname pname]=uiputfile({'*.mat','MAT-files (*.mat)'});
obj=handles.objData;
if sum(~pname)  || sum(~fname),
    return;
end
save([pname '/' fname],'obj');
disp('Save file written..');

% --- Executes on button press in Load_button.
function Load_button_Callback(hObject, eventdata, handles)
% hObject    handle to Load_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

curdir=pwd;
try
    [fname pname]=uigetfile({'*.mpd;*.tif;*.mat'},...
            'MultiSelect', 'off');
    switch lower(fname(end-2:end)),
        case 'mpd'
            handles.objData=generateObjStruct([pname fname]);
        case 'tif'
            handles.objData=generateObjStruct([pname fname]);
        case 'mat'
            temp=load([pname fname]);
            handles.objData=temp.obj;
        otherwise
            error('Unkown extention');
    end
catch e,
    cd(curdir);
    rethrow(e);
end
disp('File loaded...');
updateView(handles); %update the pretty picture

guidata(hObject, handles);
 
% --- Executes on button press in Exit_button.
function Exit_button_Callback(hObject, eventdata, handles)
% hObject    handle to Exit_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
quitq=questdlg('Are you sure you want to quit?', ...
                         'Quit confirm', ...
                         'Yes', 'No', 'Yes');

if(strncmp(quitq,'Yes',length('Yes'))==1)
    close(gcf);
end

% --- Executes on button press in accept_button.
function accept_button_Callback(hObject, eventdata, handles)
% hObject    handle to accept_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

handles.objData.boosterData{handles.objData.threshNum}(handles.objData.objNum)=1;

curmask=handles.objData.objSeg==handles.objData.objNum;
curoutline=imdilate(curmask,ones(3))-curmask;
[yind xind]=find(curoutline==1);

for j=1:length(yind),
    handles.objData.choiceMap(yind(j),xind(j),2)=1;
    handles.objData.choiceMap(yind(j),xind(j),[1 3])=0;
end

updateView(handles); %update the pretty picture

guidata(hObject, handles);

nblob_button_Callback(hObject, eventdata, handles)

% --- Executes on button press in reject_button.
function reject_button_Callback(hObject, eventdata, handles)
% hObject    handle to reject_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

handles.objData.boosterData{handles.objData.threshNum}(handles.objData.objNum)=-1;

curmask=handles.objData.objSeg==handles.objData.objNum;
curoutline=imdilate(curmask,ones(3))-curmask;
[yind xind]=find(curoutline==1);

for j=1:length(yind),
    handles.objData.choiceMap(yind(j),xind(j),1)=1;
    handles.objData.choiceMap(yind(j),xind(j),2:3)=0;
end

updateView(handles); %update the pretty picture

guidata(hObject, handles);

nblob_button_Callback(hObject, eventdata, handles)

% --- Executes on button press in djudge_button.

function djudge_button_Callback(hObject, eventdata, handles)
% hObject    handle to djudge_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

handles.objData.boosterData{handles.objData.threshNum}(handles.objData.objNum)=0;

curmask=handles.objData.objSeg==handles.objData.objNum;
curoutline=imdilate(curmask,ones(3))-curmask;
[yind xind]=find(curoutline==1);

for j=1:length(yind),
    handles.objData.choiceMap(yind(j),xind(j),1:3)=1;
end

updateView(handles); %update the pretty picture

guidata(hObject, handles);
nblob_button_Callback(hObject, eventdata, handles)

% --- Executes on button press in pblob_button.
function pblob_button_Callback(hObject, eventdata, handles)
% hObject    handle to pblob_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if (handles.objData.objNum-1)==0,
    contq=questdlg('Do you wish to move to another threshold level?', ...
                         'Quit confirm', ...
                         'Yes', 'No', 'Yes');

    if ~strncmp(contq,'Yes',length('Yes'))
        return;
    end
    
    handles.objData.objNum=1;
    if (handles.objData.threshNum-1)==0,
        handles.objData.threshNum=length(handles.objData.threshList);
    else
        handles.objData.threshNum=handles.objData.threshNum-1;
    end
  
    handles.objData.objSeg=...
        bwlabel(handles.objData.scoreIm<handles.objData.threshList(handles.objData.threshNum));
    handles.objData.objTot=max(handles.objData.objSeg(:));
    try
        if isempty(handles.objData.boosterData{handles.objData.threshNum}),

            handles.objData.boosterData{handles.objData.threshNum}=zeros(handles.objData.objTot,1);
        end
    catch e
        disp(e);
        disp('Don''t worry... live happy');
        handles.objData.boosterData{handles.objData.threshNum}=zeros(handles.objData.objTot,1);
    end
else
   handles.objData.objNum=handles.objData.objNum-1; 
end

osize=sum(sum(single(handles.objData.objSeg==handles.objData.objNum)));
if osize<30 || osize>300,  
    handles.objData.boosterData{handles.objData.threshNum}(handles.objData.objNum)=-2;
    guidata(hObject, handles);
    pblob_button_Callback(hObject, eventdata, handles);
    
else

    updateView(handles); %update the pretty picture
    guidata(hObject, handles);
end


% --- Executes on button press in nblob_button.
function nblob_button_Callback(hObject, eventdata, handles)
% hObject    handle to nblob_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


if (handles.objData.objNum+1)>handles.objData.objTot,
    contq=questdlg('Do you wish to move to another threshold level?', ...
                         'Quit confirm', ...
                         'Yes', 'No', 'Yes');

    if ~strncmp(contq,'Yes',length('Yes'))
        return;
    end
    handles.objData.objNum=1;
    if (handles.objData.threshNum+1)>length(handles.objData.threshList),
        handles.objData.threshNum=1;
    else
        handles.objData.threshNum=handles.objData.threshNum+1;
    end
  
    handles.objData.objSeg=...
        bwlabel(handles.objData.scoreIm<handles.objData.threshList(handles.objData.threshNum));
    handles.objData.objTot=max(handles.objData.objSeg(:));
    try
        if isempty(handles.objData.boosterData{handles.objData.threshNum}),

            handles.objData.boosterData{handles.objData.threshNum}=zeros(handles.objData.objTot,1);
        end
    catch e
        disp(e);
        disp('Don''t worry... live happy');
        handles.objData.boosterData{handles.objData.threshNum}=zeros(handles.objData.objTot,1);
    end
else
   handles.objData.objNum=handles.objData.objNum+1; 
end

osize=sum(sum(single(handles.objData.objSeg==handles.objData.objNum)));
if osize<30 || osize>300,  
    handles.objData.boosterData{handles.objData.threshNum}(handles.objData.objNum)=-2;
    guidata(hObject, handles);
    nblob_button_Callback(hObject, eventdata, handles);
    
else

    updateView(handles); %update the pretty picture
    guidata(hObject, handles);
end




